@@ -1,6 +1,6 @@
 package OpenGuides::Search;
 use strict;
-our $VERSION = '0.10';
+our $VERSION = '0.13';
 
 use CGI qw( :standard );
 use Wiki::Toolkit::Plugin::Locator::Grid;
@@ -118,6 +118,34 @@ precedence over C<return_output>.
 These two parameters exist to make testing easier; you probably don't
 want to use them in production.
 
+You can also request just the raw search results:
+
+  my %results = $search->run(
+                              os_x    => 528864,
+                              os_y    => 180797,
+                              os_dist => 750,
+                              format  => "raw",
+                            );
+
+Results are returned as a hash, keyed on the page name.  All results
+are returned, not just the first C<page>.  The values in the hash are
+hashes themselves, with the following key/value pairs:
+
+=over 4
+
+=item * name
+
+=item * wgs84_lat - WGS-84 latitude
+
+=item * wgs84_long - WGS-84 longitude
+
+=item * summary
+
+=item * distance - distance (in metres) from origin, if origin exists
+
+=item * score - relevance to search string, if search string exists; higher score means more relevance
+
+=back
 
 In case you're struggling to follow the code, it does the following:
 1) Processes the parameters, and bails out if it hit a problem with them
@@ -140,6 +168,11 @@ sub run {
     $self->{return_output}  = $args{return_output}  || 0;
     $self->{return_tt_vars} = $args{return_tt_vars} || 0;
 
+    my $want_raw;
+    if ( $args{vars}{format} && $args{vars}{format} eq "raw" ) {
+        $want_raw = 1;
+    }
+
     $self->process_params( $args{vars} );
     if ( $self->{error} ) {
         warn $self->{error};
@@ -151,7 +184,7 @@ sub run {
     my %tt_vars = (
                    format      => $args{'vars'}->{'format'},
                    ss_version  => $VERSION,
-                   ss_info_url => 'http://openguides.org/page/search_help'
+                   ss_info_url => 'http://openguides.org/search_help'
                   );
 
     my $doing_search;
@@ -184,9 +217,14 @@ sub run {
         $self->run_distance_search;
     }
 
-    # If we're not doing a search then just print the search form.
-    unless ( $doing_search ) {
-        return $self->process_template( tt_vars => \%tt_vars );
+    # If we're not doing a search then just print the search form (or return
+    # an empty hash if we were asked for raw results).
+    if ( !$doing_search ) {
+        if ( $want_raw ) {
+            return ( );
+        } else {
+            return $self->process_template( tt_vars => \%tt_vars );
+        }
     }
 
     # At this point either $self->{error} or $self->{results} will be filled.
@@ -207,6 +245,22 @@ sub run {
     #                  }
 
     my %results_hash = %{ $self->{results} || [] };
+
+    # If we were asked for just the raw results, return them now, after
+    # grabbing additional info.
+    if ( $want_raw ) {
+        foreach my $node ( keys %results_hash ) {
+            my %data = $self->wiki->retrieve_node( $node );
+            $results_hash{$node}{summary} = $data{metadata}{summary}[0];
+            my $lat  = $data{metadata}{latitude}[0];
+            my $long = $data{metadata}{longitude}[0];
+            my ( $wgs84_lat, $wgs84_long ) = OpenGuides::Utils->get_wgs84_coords( latitude => $lat, longitude => $long, config => $self->config );
+            $results_hash{$node}{wgs84_lat} = $wgs84_lat;
+            $results_hash{$node}{wgs84_long} = $wgs84_long;
+        }
+        return %results_hash;
+    }
+
     my @results = values %results_hash;
     my $numres = scalar @results;
 
@@ -449,17 +503,25 @@ sub _run_phrase_search {
         $contents_res{$node} = int( $contents_res{$node} / $num_results ) + 1;
     }
 
-    # It'll be a real phrase (as opposed to a word) if it has a space in it.
-    # In this case, dump out the nodes that don't match the search exactly.
-    # I don't know why the phrase searching isn't working properly.  Fix later.
-    if ( $phrase =~ /\s/ ) {
-        my @tmp = keys %contents_res;
-        foreach my $node ( @tmp ) {
-            my $content = $wiki->retrieve_node( $node );
+    my @tmp = keys %contents_res;
+    foreach my $node ( @tmp ) {
+        my $content = $wiki->retrieve_node( $node );
+
+        # Don't include redirects in search results.
+        if ($content =~ /^#REDIRECT/) {
+            delete $contents_res{$node};
+            next;
+        }
+        
+        # It'll be a real phrase (as opposed to a word) if it has a space in it.
+        # In this case, dump out the nodes that don't match the search exactly.
+        # I don't know why the phrase searching isn't working properly.  Fix later.
+        if ( $phrase =~ /\s/ ) {
             unless ( $content =~ /$phrase/i || $node =~ /$phrase/i ) {
                 delete $contents_res{$node};
-	    }
+            }
         }
+
     }
 
     my %results = map { $_ => { name => $_, score => $contents_res{$_} } }
@@ -604,6 +666,7 @@ sub process_params {
     delete $self->{y};
     delete $self->{distance_in_metres};
     delete $self->{search_string};
+    delete $self->{results};
 
     # Strip out any non-digits from distance and OS co-ords.
     foreach my $param ( qw( os_x os_y osie_x osie_y
@@ -768,7 +831,7 @@ The OpenGuides Project (openguides-dev@lists.openguides.org)
 
 =head1 COPYRIGHT
 
-     Copyright (C) 2003-2006 The OpenGuides Project.  All Rights Reserved.
+     Copyright (C) 2003-2008 The OpenGuides Project.  All Rights Reserved.
 
 The OpenGuides distribution is free software; you can redistribute it
 and/or modify it under the same terms as Perl itself.
